#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "interface.h"
#include "support.h"
#include <dirent.h>
#include <fnmatch.h>
#include <stdio.h>
#include "rlparser.h"

#define BASEDIR "/etc/initng"

void new_runlevel(GtkWidget *any_widget, const gchar *rl_name) {
  struct rl_struct *this_rl_struct = malloc(sizeof(struct rl_struct));
  gchar *filename = malloc(sizeof(rl_name));
  GtkWidget *runlevel_combo;

  runlevel_combo = lookup_widget(GTK_WIDGET(any_widget), "runlevel_combo");
  gtk_combo_box_prepend_text(GTK_COMBO_BOX(runlevel_combo), rl_name);
  
  this_rl_struct->filename = malloc(strlen(BASEDIR) + 12 +
                                      strlen(rl_name));
  strcpy(this_rl_struct->filename, BASEDIR);
  strcat(this_rl_struct->filename, "/");
  strcat(this_rl_struct->filename, rl_name);
  strcat(this_rl_struct->filename, ".runlevel");
  
  this_rl_struct->needs_saving = TRUE;
  this_rl_struct->store = gtk_list_store_new (1, G_TYPE_STRING);
  this_rl_struct->next = runlevel_struct;
  runlevel_struct = this_rl_struct;
}

void save_rl(GtkWidget * any_widget) {
  int i;
  struct rl_struct *this_rl_struct = runlevel_struct;
  GtkWidget *save_button, *save1;
  GtkTreeIter iter;
  gboolean valid;
  struct stat buff;
  FILE *file;

  save_button = lookup_widget(GTK_WIDGET(any_widget), "save_button");
  save1 = lookup_widget(GTK_WIDGET(any_widget), "save1");
  while(this_rl_struct != NULL) {
    if(this_rl_struct->needs_saving) {

      file = fopen(this_rl_struct->filename, "w");   /* Open file. */

      if (file == NULL) {
        g_print("Unable to open file %s for writing!\n", this_rl_struct->filename);
        return;
      }

      valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(this_rl_struct->store),
                                            &iter);
      while(valid) {
        gchar *str_data;
        gtk_tree_model_get(GTK_TREE_MODEL(this_rl_struct->store),
                           &iter, 0, &str_data, -1);
        fputs(str_data, file);
        fputs("\n", file);
        g_free(str_data);
        valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(this_rl_struct->store),
                                         &iter);
      }

      fclose(file);
      
      this_rl_struct->needs_saving = FALSE;
    }
    this_rl_struct = this_rl_struct->next;
  }
  gtk_widget_set_sensitive(save_button, FALSE);
  gtk_widget_set_sensitive(save1, FALSE);
}

void add_to_rl(GtkWidget * any_widget) {
  GtkWidget *runlevel_list, *service_list;
  GtkTreeIter rliter, serviceiter;
  GtkTreeSelection *selected;
  GtkTreeModel *servicemodel;
  gchar *servicename;
  GtkListStore *rlstore;
  GtkWidget *runlevel_combo, *save_button, *save1;
  gint choosen_rl;
  struct rl_struct *this_rl_struct = runlevel_struct;
  int i;

  runlevel_list = lookup_widget(GTK_WIDGET(any_widget), "runlevel_list");
  service_list = lookup_widget(GTK_WIDGET(any_widget), "service_list");

  selected = gtk_tree_view_get_selection(GTK_TREE_VIEW(service_list));
  rlstore = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(runlevel_list)));
  if(gtk_tree_selection_get_selected(selected, &servicemodel, &serviceiter)) {
    gtk_tree_model_get (servicemodel, &serviceiter, 0, &servicename, -1);
    gtk_list_store_append (rlstore, &rliter);
    gtk_list_store_set (rlstore, &rliter, 0, servicename, -1);
    g_free(servicename);
    
    save_button = lookup_widget(GTK_WIDGET(any_widget), "save_button");
    save1 = lookup_widget(GTK_WIDGET(any_widget), "save1");
    runlevel_combo = lookup_widget(GTK_WIDGET(any_widget), "runlevel_combo");  
    choosen_rl = gtk_combo_box_get_active(GTK_COMBO_BOX(runlevel_combo));
    
    for(i = 0; i < choosen_rl; i++) {
      this_rl_struct = this_rl_struct->next;
    }
    this_rl_struct->needs_saving = TRUE;
    gtk_widget_set_sensitive(save_button, TRUE);
    gtk_widget_set_sensitive(save1, TRUE);
  }
}

void remove_from_rl(GtkWidget * any_widget) {
  GtkWidget *runlevel_list;
  GtkTreeIter iter;
  GtkTreeSelection *selected;
  GtkTreeModel *model;
  GtkWidget *runlevel_combo, *save_button, *save1;
  gint choosen_rl;
  struct rl_struct *this_rl_struct = runlevel_struct;
  int i;
  
  runlevel_list = lookup_widget(GTK_WIDGET(any_widget), "runlevel_list");
  selected = gtk_tree_view_get_selection(GTK_TREE_VIEW(runlevel_list));
  if(gtk_tree_selection_get_selected(selected, &model, &iter)) {
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    
    save_button = lookup_widget(GTK_WIDGET(any_widget), "save_button");
    save1 = lookup_widget(GTK_WIDGET(any_widget), "save1");
    runlevel_combo = lookup_widget(GTK_WIDGET(any_widget), "runlevel_combo"); 
    choosen_rl = gtk_combo_box_get_active(GTK_COMBO_BOX(runlevel_combo));
    
    for(i = 0; i < choosen_rl; i++) {
      this_rl_struct = this_rl_struct->next;
    }
    this_rl_struct->needs_saving = TRUE;
    gtk_widget_set_sensitive(save_button, TRUE);
    gtk_widget_set_sensitive(save1, TRUE);
  }
}


int rlfilefilter(const struct dirent* entry) {
  if(!fnmatch("*.runlevel", entry->d_name, 0))
    return 1;
  else
    return 0;
}

void update_rllist(GtkWidget *runlevel_combo) {
  gchar *chosenrl = gtk_combo_box_get_active_text(GTK_COMBO_BOX(runlevel_combo));
  gint choosen_rl = gtk_combo_box_get_active(GTK_COMBO_BOX(runlevel_combo));
  struct stat buff;
  FILE *file;              /* File descriptor for file */
  int res;                    /* Result of read */
  GtkTreeIter iter;
  GtkWidget *runlevel_list;
  GtkWidget *remove_button, *add_button;
  int i;
  struct rl_struct *this_rl_struct = runlevel_struct;

  remove_button = lookup_widget(GTK_WIDGET(runlevel_combo), "remove_button");
  add_button = lookup_widget(GTK_WIDGET(runlevel_combo), "add_button");
  runlevel_list = lookup_widget(GTK_WIDGET(runlevel_combo), "runlevel_list");  

  gtk_widget_set_sensitive(remove_button, FALSE);

  for(i = 0; i < choosen_rl; i++) {
    this_rl_struct = this_rl_struct->next;
  }
  
  if(this_rl_struct->store == NULL) {
    this_rl_struct->store = gtk_list_store_new (1, G_TYPE_STRING);

    if(lstat(this_rl_struct->filename, &buff)<0)
      perror("Runlevel file doesn't exist!");
  
    file = fopen(this_rl_struct->filename, "rt");   /* Open file. */

    if (file == NULL) 
      return;

    /* Allocate a file buffer */
    char* buffer = malloc(64); //UGLY!

    /* Read whole file */
    while(fgets(buffer, 64, file) > 0) {
      if(strlen(buffer) > 1) {
        buffer[strlen(buffer)-1] = 0;
        gtk_list_store_append (this_rl_struct->store, &iter);
        gtk_list_store_set (this_rl_struct->store, &iter, 0, buffer, -1);
      }
    }

    gtk_widget_set_sensitive(add_button, TRUE);

    fclose(file);
    free(buffer);  
  }
  gtk_tree_view_set_model(GTK_TREE_VIEW(runlevel_list),
                          GTK_TREE_MODEL(this_rl_struct->store));
}

void populate_runlevel_combo(GtkWidget * any_widget) {
  GtkWidget *runlevel_combo;
  runlevel_struct = malloc(sizeof(struct rl_struct));
  struct rl_struct *this_rl_struct = runlevel_struct;
  int i;

  runlevel_combo = lookup_widget(GTK_WIDGET(any_widget), "runlevel_combo");

  struct dirent **namelist;
  int n = scandir(BASEDIR, &namelist, &rlfilefilter, alphasort);
  if(n < 0)
    perror("Unable to read directory");

  gtk_combo_box_new_text();
  for(i = 0; i < n; i++) {
    this_rl_struct->filename = malloc(strlen(BASEDIR) + 2 +
                                      strlen(namelist[i]->d_name));
    strcpy(this_rl_struct->filename, BASEDIR);
    strcat(this_rl_struct->filename, "/");
    strcat(this_rl_struct->filename, namelist[i]->d_name);
    
    namelist[i]->d_name[strlen(namelist[i]->d_name)-9] = 0;
    gtk_combo_box_append_text(GTK_COMBO_BOX(runlevel_combo),
                              namelist[i]->d_name);
    free(namelist[i]);
    this_rl_struct->needs_saving = FALSE;
    this_rl_struct->store = NULL;
    if(i < n-1) {
      this_rl_struct->next = malloc(sizeof(struct rl_struct));
      this_rl_struct = this_rl_struct->next;
    } else {
      this_rl_struct->next = NULL;
    }
  }
  free(namelist);
}

void init_runlevel_list(GtkWidget * any_widget) {
  GtkWidget *runlevel_list;
  GtkCellRenderer *rlrenderer;

  rlrenderer = gtk_cell_renderer_text_new();
  runlevel_list = lookup_widget(GTK_WIDGET(any_widget), "runlevel_list");
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(runlevel_list),
                                              -1, "name",
					      rlrenderer, "text", 0,
					      NULL);
}
