#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "serviceparser.h"
#include "interface.h"
#include "support.h"
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <assert.h>
#include <fnmatch.h>
#include <stdio.h>

#define BASEDIR "/etc/initng"

int filefilter(const struct dirent* entry) {
  if(strcmp(entry->d_name, ".") && strcmp(entry->d_name, ".."))
    return 1;
  else
    return 0;
}

void recursive_scan(char* path, GtkListStore *store) {
  GtkTreeIter iter;
  struct s_info *serviceinfo;
  
  struct dirent **namelist;
  int n = scandir(path, &namelist, &filefilter, alphasort);
  if(n < 0)
    perror("Unable to read directory");
  else {
    while(n--) {
      struct stat buff;
      char *fullpath = malloc(strlen(path) + strlen(namelist[n]->d_name) + 2);
      sprintf(fullpath, "%s/%s", path, namelist[n]->d_name);
      if(lstat(fullpath, &buff)<0)
        perror("Strange fault!");
      if(S_ISDIR(buff.st_mode))
        recursive_scan(fullpath, store);
      else if(!fnmatch("*.i", namelist[n]->d_name, 0)) {
        FILE *file = fopen(fullpath, "rt");   /* Open file. */

        if (file == NULL) 
          continue;

        char* buffer = malloc(255); //UGLY!
        while(fgets(buffer, 255, file) > 0) {
          if(strstr(buffer, "daemon") == buffer) {
            serviceinfo = malloc(sizeof(struct s_info));
            serviceinfo->startline = malloc(strlen(buffer) + 1);
            strcpy(serviceinfo->startline, buffer);
            char *tok = strtok(buffer, " ");
            tok = strtok(NULL, " ");
            gtk_list_store_prepend (store, &iter);
            gtk_list_store_set (store, &iter, 0, tok, 1, "daemon", -1);
            serviceinfo->name = tok;
            serviceinfo->type = DAEMON;
            serviceinfo->path = fullpath;
            g_object_set_data(G_OBJECT(store), tok, serviceinfo);
          } else if(strstr(buffer, "service") == buffer) {
            serviceinfo = malloc(sizeof(struct s_info));
            serviceinfo->startline = malloc(strlen(buffer) + 1);
            strcpy(serviceinfo->startline, buffer);
            char *tok = strtok(buffer, " ");
            tok = strtok(NULL, " ");
            gtk_list_store_prepend (store, &iter);
            gtk_list_store_set (store, &iter, 0, tok, 1, "service", -1);
            serviceinfo->name = tok;
            serviceinfo->type = SERVICE;
            serviceinfo->path = fullpath;
            g_object_set_data(G_OBJECT(store), tok, serviceinfo);          
          } else if(strstr(buffer, "virtual") == buffer) {
            serviceinfo = malloc(sizeof(struct s_info));
            serviceinfo->startline = malloc(strlen(buffer) + 1);
            strcpy(serviceinfo->startline, buffer);
            char *tok = strtok(buffer, " ");
            tok = strtok(NULL, " ");
            gtk_list_store_prepend (store, &iter);
            gtk_list_store_set (store, &iter, 0, tok, 1, "virtual", -1);
            serviceinfo->name = tok;
            serviceinfo->type = VIRTUAL;
            serviceinfo->path = fullpath;
            g_object_set_data(G_OBJECT(store), tok, serviceinfo);          
          }
        }
        fclose(file);
        free(buffer);
      } else if(!fnmatch("*.runlevel", namelist[n]->d_name, 0)) {
        serviceinfo = malloc(sizeof(struct s_info));
        namelist[n]->d_name[strlen(namelist[n]->d_name)-9] = 0;
        gtk_list_store_prepend (store, &iter);
        gtk_list_store_set(store, &iter, 0, namelist[n]->d_name, 1, "runlevel", -1);
        serviceinfo->name = namelist[n]->d_name;
        serviceinfo->type = RUNLEVEL;
        serviceinfo->path = fullpath;
        serviceinfo->startline = NULL;
        g_object_set_data(G_OBJECT(store), namelist[n]->d_name, serviceinfo);
      }
      free(namelist[n]);
    }
    free(namelist);
  }
}

void load_service(GtkWidget *initconf_show_service, GtkWidget *treeview) {
  GtkWidget *service_textview;
  GtkTextBuffer *buffer;
  GtkTreeSelection *selected;
  GtkTreeIter iter;
  GtkTreeModel *model;
  gchar *servicename, *filename;
  struct stat buff;
  FILE *file;
  gchar res;
  int depth = 1;
  
  service_textview = lookup_widget(initconf_show_service, "service_textview");
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(service_textview));
  selected = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
  
  gtk_tree_selection_get_selected(selected, &model, &iter);
  gtk_tree_model_get (model, &iter, 0, &servicename, -1);
  struct s_info *serviceinfo = g_object_get_data(G_OBJECT(model), servicename);
  filename = serviceinfo->path;

  if(lstat(filename, &buff)<0)
    perror("Strange fault!");
  
  file = fopen(filename, "rt");
  if (file == NULL) 
    return;
  
  if(serviceinfo->type != RUNLEVEL) {
    char* temp = malloc(255); //UGLY!
    while(fgets(temp, 255, file) > 0) {
      if(strcmp(temp, serviceinfo->startline) == 0) {
        break;
      }
    }
    free(temp);
  }
  
  gtk_text_buffer_set_text(buffer, "", 0);

  do {
    res = fgetc(file);
    if(res == '{') {
      depth++;
    } else if(res == '}') {
      depth--;
    }
    if(depth <= 0 || res == EOF)
      break;
    gtk_text_buffer_insert_at_cursor(buffer, &res, 1);
  } while(TRUE);
  
  g_free(servicename);
}

void populate_service_list(GtkWidget * any_widget) {

  GtkWidget *service_list;
  GtkCellRenderer *servicerenderer;
  GtkTreeModel *servicemodel;
  GtkListStore  *servicestore = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);

  service_list = lookup_widget(GTK_WIDGET(any_widget), "service_list");
  servicerenderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(service_list),
                                              -1, "name",
					      servicerenderer, "text", 0,
					      NULL);
  servicerenderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(service_list),
                                              -1, "function",
					      servicerenderer, "text", 1,
					      NULL);
  recursive_scan(BASEDIR, servicestore);
  servicemodel = GTK_TREE_MODEL(servicestore);
  gtk_tree_view_set_model(GTK_TREE_VIEW(service_list), servicemodel);
  
  g_object_unref(servicemodel);
}
